// FILE: main.c

/* simulates the probability of ruin using the regenerative approach */

// Mik Cieslak and Kris Luttmer

#include "qmc.h"
#include "ruinREG.h"

#define NUMBLOCKS 2
#define INTLOOP 0
#define EXTLOOP 1

int ReadUserInput (char *filename, double *lambda, double *mu, double *c,
		   double *delta, double *u);

// -------------------------------------------------------------------------

int main(int argc, char **argv)
{  
  int i, j;     // looping variables
  FILE *infile; // the input file
  int m, n;     // number of randomizations and number of points
  
  double *point = NULL;   // a point in the point set
  double *rvector = NULL; // the randomization vector
  double *rpoint = NULL;  // the randomized point
  
  STATSQMC stats = NULL; // statistics accumulator
  CONFIDENCEINTERVAL cf; // the confidence interval

  // user variables for estimating the probability of ruin
  double lambda, mu, c, delta, u;
  double WBarre, TBarre, cov, mu_hat, sigmav2, ectype;
  double D, sumTi;
 
  // check for correct number of arguments
  if(argc != 3)
    {
      printf ("\n\tNo input file(s) Specified\n");
      printf ("\tUsage: %s filename1 filename2\n", argv[0]);
      printf ("\tfilename1 = QMC parameters input file\n");
      printf ("\tPlease run MAKEINPUT or specify filename\n");
      printf ("\tfilename2 = ruinREG parameters input file\n\n");
      return (0);
    }

  if(!LoadInputQMC(argv[1]))
    {
      printf ("Cannot load the input file.\n");
      return (0);
    }     

  // read the user input file
  if (!ReadUserInput (*(argv+2), &lambda, &mu, &c, &delta, &u))
    {
      printf ("Cannot read the user input file.");
      return (0);
    }

  // initialize the method according to the parameters 
  if (!InitQMC ())
    {
      printf ("Cannot initialize the Quasi Monte Carlo method.\n");
      return (0);
    }
 
  // init the stats blocks  
  if ((stats = InitStat (NUMBLOCKS)) == NULL)
    {
      printf ("Cannot initialize stats.\n");
      FreeQMC();
      return (0);
    }
   
  // for the number of randomizations, do

  m = GetQMC(QMC_RANDS);
  n = GetQMC(QMC_NPOINTS);
  printf("n=%d",n);

  for (i = 0; i < m; i++)
    {     
      ResetStat (stats, INTLOOP);
      ResetQMC();

      rvector = GenRandom(); // generate the random vector

      // for the number of points in the point set
      for (j = 0; j < n; j++)
	{
	  // generate the point using the QMC method
	  if ((point = QMC()) == NULL)
	    {
	      printf ("Cannot generate point %d.\n", j);
	      FreeQMC();
	      FreeRandom();
	      FreeStat (stats);
	      return (0);
	    }
	  
	  // apply the Randomization
	  if ((rpoint = AppRandom (point, rvector)) == NULL)
	    {
	      printf ("Cannot apply the randomization\n");
	      FreeQMC();
	      FreeRandom();
	      FreeStat (stats);
	      return (0);
	    }
	 
	  // call a user function for now we will just call the asian option
	  RuinREG (lambda, mu, c, delta, u, &D, &sumTi);
	    
	  // Update inner loop stats
          StatUpdate2 (stats, INTLOOP, D, sumTi);
	}      

      // update the exterior loop
      D = StatAverage (stats, INTLOOP, RANDVARONE);
      sumTi = StatAverage (stats, INTLOOP, RANDVARTWO);
      StatUpdate2 (stats, EXTLOOP, D, sumTi);
    }
  
  WBarre = StatAverage (stats, EXTLOOP, RANDVARONE);
  TBarre = StatAverage (stats, EXTLOOP, RANDVARTWO);
  mu_hat = WBarre / TBarre;

  printf ("Average: %f\n", mu_hat);
  
  cov = StatCovariance (stats, EXTLOOP);

  sigmav2 = StatVariance (stats, EXTLOOP, RANDVARONE) + 
    (mu_hat * mu_hat * StatVariance (stats, EXTLOOP, RANDVARTWO)) -
    (2.0 * mu_hat * cov);

  ectype = sqrt (sigmav2 / ((double) m)) / TBarre;

  printf ("Variance: %f\n", ectype);

  // free the randomization method, the QMC method, and the stats blocks
  FreeQMC(); 
  FreeStat (stats);
  return (1);
}

// ----------------------------------------------------------------------

int ReadUserInput (char *filename, double *lambda, double *mu, double *c,
		   double *delta, double *u)
{
  int i;
  FILE *infile;
  float temp;

  if (!filename)
    return (0);

  if ((infile = fopen (filename, "r")) == NULL)
    return (0);

  fscanf (infile, "%f", &temp);
  *lambda = (double) temp;
  fscanf (infile, "%f", &temp);
  *mu = (double) temp;
  fscanf (infile, "%f", &temp);
  *c = (double) temp;
  fscanf (infile, "%f", &temp);
  *delta = (double) temp; 
  fscanf (infile, "%f", &temp);
  *u = (double) temp;
  
  fclose (infile);
  return (1);
}

// ---------------------------------------------------------------
